//includes files and libraries needed to this project
#include <Servo.h>
#include "pitches.h"

//global constants (such as pins)
#define gled 10
#define rled 9
#define restartPin 13
#define restartBtn 8
#define piezo 12
#define servo 11
#define btnNum 6
#define deb_delay 50

//relevant to general pushbutton status
int btnArray [btnNum] = {2, 3, 4, 5, 6, 7};
int goodArray [btnNum-1];
bool statArray [btnNum-1] = {HIGH, HIGH, HIGH, HIGH, HIGH};
bool prevstatArray [btnNum-1] = {LOW, LOW, LOW, LOW, LOW};

//relevant to restart button
bool statRestart = HIGH; 
bool prevstatRestart = LOW;

//holds pressed (P) or unpressed (U) status of the pushbuttons
String pressArray [btnNum-1] = {"U", "U", "U", "U", "U"};

//relevant to "bad" pushbutton status
int badPin;
bool statBad = HIGH;
bool prevstatBad = LOW;

//this variable holds number of buttons already pressed
int pressCount = 0; 

//relevant to LED status and blinking
int gledStat = LOW;
int rledStat = LOW;
unsigned long prevMillis = 0;
unsigned long interval = 1000; //blink interval

//relevant to servo
Servo serv;
int posI = 0;
int posF = 45;

//relevant to piezo melody
int melody [] = {NOTE_C4, NOTE_G3, NOTE_G3, NOTE_A3, NOTE_G3, 0, NOTE_B3, NOTE_C4};
int noteDurations [] = {4, 8, 8, 4, 4, 4, 4, 4};

void setup() {
  Serial.begin(9600);

  digitalWrite(restartPin, LOW);
  pinMode(restartPin, OUTPUT);

  serv.attach(servo);

  pinMode(gled, OUTPUT);
  pinMode(rled, OUTPUT);
	
	pinMode(restartBtn, INPUT_PULLUP);

  for (int i =0; i< btnNum; i++){
    pinMode(btnArray[i], INPUT_PULLUP);
  }

	/*create random seeed from unused analog pin noise,
	will generate random sequence when restarted*/
  randomSeed(analogRead(A0));

  //set randomized "bad" button pin within pushbutton array
  badPin = btnArray[random(btnNum-1)];

  //create new array excluding the bad pin
  for (int i = 0; i< btnNum; i++){
    if(btnArray[i] != badPin){
      int j = 0;
      goodArray[j] = btnArray[i];
      j++;
    }
  }
  Serial.println(badPin);
}

void loop() {
  for(int i = 0; i< btnNum-1; i++){
    if(digitalRead(goodArray[i]) == LOW && statArray[i] != prevstatArray[i] && pressArray[i] == "U"){
			delay (deb_delay); //can be changed to milis function for non-blocking code
      pressCount++;
			/*this statement impedes the counter form registering
			the same button as being pressed more than once*/
			pressArray[i] = "P";
      digitalWrite(gled, HIGH);
      tone(piezo, 1000, 500);
      delay(deb_delay);
      digitalWrite(gled, LOW);
			prevstatArray[i] = !statArray[i]; //debounce function
    }    
	}if(digitalRead(badPin) == LOW && statBad != prevstatBad && pressCount >= (btnNum-1)){
      delay(deb_delay);
			gameWin();
			prevstatBad = !statBad;

  }else if(digitalRead(badPin) == LOW && statBad != prevstatBad && pressCount < (btnNum-1)){
      delay(deb_delay);
			gameOver();
  }
  
}

//runs if game is won
void gameWin(){
 digitalWrite(rled, LOW);
 unsigned long currentMillis =  millis();
	if (currentMillis - prevMillis >= interval){
	  prevMillis = currentMillis;

    if(gledStat == LOW){
     gledStat = HIGH;

    }else{
	    gledStat = LOW;
    }
  }
  //plays piezo melody
 for (int thisNote = 0; thisNote < 8; thisNote++) {
    int noteDuration = 1000 / noteDurations[thisNote];
    tone(8, melody[thisNote], noteDuration);
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
    noTone(8);
  }
  serv.write(posI);
	digitalWrite(gled, gledStat);
	//add piezo & servo
}

//runs if game is lost
void gameOver(){
 digitalWrite(gled, LOW);
 unsigned long currentMillis =  millis();
	if(currentMillis - prevMillis >= interval ){
	  prevMillis = currentMillis;

    if(rledStat == LOW){
     rledStat = HIGH;

    }else{
	    rledStat = LOW;
    }
  }
  tone(piezo, 500, 1000);
	digitalWrite(rled, rledStat);
	//add piezo & servo
}

//restarts the game
void reStart() {
  if(restartBtn == LOW && statRestart != prevstatRestart){
    serv.write(posF);
    delay(1000);
    digitalWrite(restartPin, HIGH);
  }
}